static char sccsid[] = "%Z%%M%  %I%  %W% %G% %U%";
/*
 *   (C) COPYRIGHT International Business Machines Corp. 1999
 *   All Rights Reserved
 *   Licensed Materials - Property of IBM
 *   US Government Users Restricted Rights - Use, duplication or
 *   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/*
 *   Translation Command
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "kingapi.h"
#include "transenv.h"

#define VERSION			"1.0.0.17"

void usage(void)
{
    const char* msg =
	"Usage: kingtr [options] [file]\n"
	"         -t   specify either 'text' or 'html' (default is text)\n"
	"         -e   specify english sentence to translate\n"
	"         -d   specify system or user dictionaries\n"
	"         -s   specify a path of system dictionary\n"
	"         -o   specify output filename\n"
	"         -help      produce this help message\n"
	"         -version   report kingtr version\n";
    printf(msg);
    exit(0);
}

int
main(int argc, char* argv[])
{
    int html_mode = 0;
    char* english = NULL;
    char* sysdic = NULL;
    char* dicts = NULL;
    char* outfile = NULL;
    char* infile = NULL;

    TransEnv env;
    if (!read_kingrc(USER_CONFIG_FILE, &env))
	(void)read_kingrc(SYSTEM_CONFIG_FILE, &env);

    while (--argc > 0) {
	char* p = *++argv;
	if (!strcmp(p, "-t")) {
	    p = *++argv;
	    argc--;
	    if (!strcmp(p, "text"))
		html_mode = 0;
	    else if (!strcmp(p, "html"))
		html_mode = 1;
	    else
		usage();
	}

	else if (!strcmp(p, "-e")) {
	    english = *++argv;
	    argc--;
	    if (!english || !*english)
		usage();
	}

	else if (!strcmp(p, "-s")) {
	    sysdic = *++argv;
	    argc--;
	    if (!sysdic || !*sysdic)
		usage();
	    if (env.SystemDictDirectory != NULL)
		delete[] env.SystemDictDirectory;
	    env.SystemDictDirectory = expand_file_name(sysdic);
	}

	else if (!strcmp(p, "-d")) {
	    dicts = *++argv;
	    argc--;
	    if (!dicts || !*dicts)
		usage();
	    if (env.Dictionaries != NULL)
		delete[] env.Dictionaries;
	    env.Dictionaries = expand_file_names(dicts);
	}

	else if (!strcmp(p, "-o")) {
	    outfile = *++argv;
	    argc--;
	    if (!outfile || !*outfile)
		usage();
	}

	else if (!strcmp(p, "-h") || !strcmp(p, "-help")
		 || !strcmp(p, "--help")) {
	    usage();
	}

	else if (!strcmp(p, "-version") || !strcmp(p, "--version")) {
	    printf("kingtr version %s\n", VERSION);
	    exit(0);
	}

	else if (p[0] == '-') {
	    printf("Invalid option %s\n", p);
	    usage();
	}

	else
	    infile = p;
    }

    char* buf = NULL;
    if (infile) {
	struct stat sbuf;
	if (stat(infile, &sbuf) < 0) { // file not found
	    fprintf(stderr, "%s not found.\n", infile);
	    exit(1);
	}
	int size = sbuf.st_size;

	int fdi;
	if ((fdi = open(infile, O_RDONLY)) < 0) {
	    fprintf(stderr, "Can't open %s for reading.\n", infile);
	    exit(1);
	}

	buf = new char[size + 1];
	if (!buf) {
	    fprintf(stderr, "Memory exhausted.\n");
	    close(fdi);
	    exit(1);
	}
	if (read(fdi, buf, size) != size) {
	    fprintf(stderr, "%s: read failed.\n", infile);
	    close(fdi);
	    delete[] buf;
	    exit(1);
	}
	buf[size] = '\0';
	close(fdi);
    }
    else if (english) {
	buf = new char[strlen(english) + 1];
	strcpy(buf, english);
    }
    else {
	int n;
	const int buf_siz = 1024;
	int size = buf_siz;
	buf = new char[size + 1];
	char* bufp = buf;
	while ((n = read(0, bufp, buf_siz)) > 0) {
	    bufp[n] = '\0';
	    bufp += n;
	    if (n == buf_siz && size - (bufp - buf) < buf_siz) {
		size <<= 1;
		char* tmp = new char[size + 1];
		strcpy(tmp, buf);
		delete[] buf;
		buf = tmp;
		bufp = buf + strlen(buf);
	    }
	}
    }

    char** groups = NULL;
    if (env.Dictionaries) {
	groups = new char*[128];
	char* p = env.Dictionaries;
	int i;
	for (i = 0; p; i++) {
	    groups[i] = p;
	    p = strchr(p, ',');
	    if (p)
		*p++ = '\0';
	}
	groups[i] = NULL;
    }
    void* id = KingInit(env.SystemDictDirectory, groups);
    delete[] groups;
    if (!id) {
	fprintf(stderr, "%s\n", KingGetLastError());
	exit(1);
    }
    if (env.ParamThink) {
	if (!strcmp(env.ParamThink, "wordonly"))
	    KingSetParam(id, KING_PARAM_THINK, KING_THINK_WORDONLY);
	else if (!strcmp(env.ParamThink, "shallow"))
	    KingSetParam(id, KING_PARAM_THINK, KING_THINK_SHALLOW);
	else if (!strcmp(env.ParamThink, "default"))
	    KingSetParam(id, KING_PARAM_THINK, KING_THINK_DEFAULT);
	else if (!strcmp(env.ParamThink, "normal"))
	    KingSetParam(id, KING_PARAM_THINK, KING_THINK_NORMAL);
	else if (!strcmp(env.ParamThink, "deep"))
	    KingSetParam(id, KING_PARAM_THINK, KING_THINK_DEEP);
    }
    if (env.ParamStyle) {
	if (!strcmp(env.ParamStyle, "normal"))
	    KingSetParam(id, KING_PARAM_STYLE, KING_STYLE_NORMAL);
	else if (!strcmp(env.ParamStyle, "formal"))
	    KingSetParam(id, KING_PARAM_STYLE, KING_STYLE_FORMAL);
    }
    if (env.ParamPunct) {
	if (!strcmp(env.ParamPunct, "jpn"))
	    KingSetParam(id, KING_PARAM_PUNCT, KING_PUNCT_JPN);
	else if (!strcmp(env.ParamPunct, "eng"))
	    KingSetParam(id, KING_PARAM_PUNCT, KING_PUNCT_ENG);
    }
    if (env.ParamNum) {
	if (!strcmp(env.ParamNum, "zenkaku"))
	    KingSetParam(id, KING_PARAM_NUM, KING_NUM_ZENKAKU);
	else if (!strcmp(env.ParamNum, "hankaku"))
	    KingSetParam(id, KING_PARAM_NUM, KING_NUM_HANKAKU);
    }
    if (env.ParamUnitConv) {
	if (!strcmp(env.ParamUnitConv, "on"))
	    KingSetParam(id, KING_PARAM_UNITCONV, KING_UNITCONV_ON);
	else if (!strcmp(env.ParamUnitConv, "off"))
	    KingSetParam(id, KING_PARAM_UNITCONV, KING_UNITCONV_OFF);
    }
    if (env.ParamKanji) {
	if (!strcmp(env.ParamKanji, "jis"))
	    KingSetParam(id, KING_PARAM_KANJI, KING_KANJI_JIS);
	else if (!strcmp(env.ParamKanji, "sjis"))
	    KingSetParam(id, KING_PARAM_KANJI, KING_KANJI_SJIS);
	else if (!strcmp(env.ParamKanji, "euc"))
	    KingSetParam(id, KING_PARAM_KANJI, KING_KANJI_EUC);
    }

    char* retdata;
    int retsize;
    if (html_mode == 0)
	KingTranslateText(id, buf, strlen(buf), &retdata, &retsize);
    else /* if (html_mode == 1) */ {
	int style;
	if (!strcmp(env.OutputStyle, "en_jp"))
	    style = 1;
	else /* if (!strcmp(env.OutputStyle, "jp")) */
	    style = 0;
	KingTranslateHtml(id, buf, strlen(buf), &retdata, &retsize, style);
    }
    KingExit(id);

    if (outfile) {
	int fdo;
	if ((fdo = open(outfile, O_CREAT|O_WRONLY, 0644)) < 0) {
	    fprintf(stderr, "Can't open %s for writing.\n", outfile);
	    exit(1);
	}
	if (write(fdo, retdata, retsize) != retsize) {
	    fprintf(stderr, "%s: write failed.\n", outfile);
	    close(fdo);
	    exit(1);
	}
	write(fdo, "\n", 1);
	close(fdo);
    }
    else
	printf("%s\n", retdata);
    delete[] buf;
    delete[] retdata;
    return 0;
}
